import { z } from "zod";
/**
 * `HeaderValue` type represents what can be used to add HTTP header values.
 * This type follows the `headers` type required by the fetch API itself.
 *
 * Note that this is NOT A JSON stringifiable type, since headers for the fetch
 * API do not accept arbitrary objects.
 *
 * Exporting this type so that you can explicitly type your Header objects
 * with this to ensure that it is correctly typed at point of value definition
 * instead of only type checking when you call the `.header` method.
 *
 * Note that although traditionally primitive values like numbers and boolean
 * were allowed, it is only because they were implicitly converted to strings
 * before they were sent out, as the HTTP spec requires header values to be
 * strings. Thus this stricter type makes it more obvious and enforces it at the
 * user level.
 *
 * https://stackoverflow.com/questions/34152142/possible-types-of-a-http-header-value
 */
type HeaderValue = Exclude<HeadersInit, Headers>;
/**
 * ## `Header` Type
 * Header can either be,
 * 1. A `HeaderValue`.
 * 2. A function that returns a `HeaderValue` or undefined.
 * 3. A function that returns a Promise that resolves to `HeaderValue` or
 * undefined.
 *
 * Header cannot be `undefined` directly because the `header` method cannot be
 * directly called with `undefined` because that wouldn't make any sense.
 *
 * Exporting this type so that library users can explicitly type Header values
 * with this to ensure that it is correctly typed at point of value definition
 * instead of only type checking when you call the `.header` method.
 *
 * ## `Header` function types
 * The function types can return undefined instead of a `HeaderValue` because
 * the API call should still run even if the header cannot be generated. For
 * e.g. if using a function that reads auth token from local storage and only
 * return a `HeaderValue` with the token if it exists. This will not cause any
 * errors because spreading `undefined` into an object will just do nothing
 * `{ something: true, ...undefined }` will just be `{ something: true }`
 *
 * If function throws, the API call will be cancelled and the exception will be
 * caught by the `safe` function wrapper before they are bubbled up and returned
 * to the library user.
 */
type Header = HeaderValue | (() => HeaderValue | undefined) | (() => Promise<HeaderValue | undefined>);
/**
 * All the supported HTTP methods.
 * Ref: https://fetch.spec.whatwg.org/#methods
 *
 * Based on the reference link, although there are other HTTP methods, they are
 * not supported by the `fetch` spec, and therefore not included in this union
 * type as even if you could pass it to the `fetch` function, it will just fail.
 *
 * `HEAD` and `OPTIONS` HTTP methods are quite special and low level and rarely
 * used, therefore these are the only 2 methods which do not have wrapper
 * methods to easily create `Fetch` instances from `Builder` instance. However
 * library users can simply use the low level `HTTP` method on `Builder`.
 *
 * Exporting this type so that you can explicitly type your HTTP method strings
 * as needed to ensure that they are correctly typed at point of value
 * definition instead of only type checking when you call the constructor.
 */
type HTTPMethod = "GET" | "POST" | "PUT" | "PATCH" | "DELETE" | "HEAD" | "OPTIONS";
/**
 * This is an alias for the `any` type because JSON parsing and stringification
 * can support virtually any type.
 */
type JsonTypeAlias = any;
/**
 * Custom named exception class so that library users can check for this failure
 * mode with the `instanceof` operator.
 *
 * This exception will be thrown when there is a custom timeout value set, and
 * the API did not respond before the time.
 */
declare class TimeoutException extends Error {
}
/**
 * Custom named exception class so that library users can check for this failure
 * mode with the `instanceof` operator.
 *
 * This exception is used when there is an exception thrown during Header
 * generation process right before the API call. In `Fetch.#fetch` method,
 * header values are generated, where functions and async functions are accepted
 * as header values to generate header objects right before the API call. Any
 * exceptions thrown are caught, and wrapped with this `HeaderException` type
 * before being returned to the library user.
 *
 * ## Why wrap the original exception?
 * Since these functions can throw any possible exception types, it is hard for
 * library users to pin down the exact cause of the exception using `instanceof`
 * operator, especially so when the header generation function they use is not
 * authored by them and can throw arbitrary exception types (e.g. using an auth
 * library and that throws some custom exception).
 *
 * By wrapping it in this `HeaderException` Error object type, the library user
 * can easily determine that header generation is what caused the exception with
 * a `instanceof` check. If the user would like to drill down on the specifics,
 * they can continue to do so as the original exception thrown is kept on the
 * `error` property of the `HeaderException` instance. Use `instanceof` operator
 * on `HeaderException.error` to continue type narrowing down the root cause.
 *
 * Example of drilling down on the root cause
 * ```typescript
 * class MyCustomError extends Error {}
 *
 * const { res, err } = await sf
 *     .useOnce("https://example.com/test")
 *     .GET()
 *     .useHeader(async () => { throw new MyCustomError("some failure"); })
 *     .runJSON();
 *
 * if (err instanceof HeaderException && err.error instanceof MyCustomError) {
 *      console.log("API call failed as Header function threw MyCustomError");
 * }
 * ```
 */
declare class HeaderException extends Error {
    readonly error: Error;
    constructor(/**
     * The original exception thrown during the header generation process. This
     * is stored so that library users can use this to check root cause of the
     * issue using the `instanceof` operator.
     */
    error: Error);
}
/**
 * Custom named exception class so that library users can check for this failure
 * mode with the `instanceof` operator.
 *
 * This exception will be used when the API call's response data fails runtime
 * response validation with the custom validator the library user passed in.
 * This will only be thrown if the validator returned false and did not throw an
 * exception. In the case where the Validator throws an exception, that will be
 * returned to the library user instead of this.
 *
 * ## Potential add on
 * Potentially, the exception name can be changed to the class name so that if
 * library users ever log the exception out, `exception.toString()` method will
 * use the custom exception name as part of the string, which can help users
 * with debugging it by looking at the exception name instead of just having it
 * use the generic `Error` name. However weighing this usefulness against
 * library size, it is better to have a smaller library since this is not a need
 * to have feature as library users should already be using `instanceof`
 * operator to check exception type in the first place.
 *
 * Example
 * ```typescript
 * export class ValidationException extends Error {
 *   constructor(message: string) {
 *     super(message);
 *     this.name = "ValidationException";
 *   }
 * }
 * ```
 */
declare class ValidationException extends Error {
}
/**
 * Union type for all 'simpler-fetch' defined exceptions that could be
 * **returned** from an API call. This covers all possible exceptions from
 * network failure to exceptions thrown from header function(s), to runtime
 * response validation failure.
 *
 * Note that this only applies to exceptions and not errors, where exceptions
 * are error states that are potentially expected, and should be dealt with
 * sequentially as part of a possible state after executing a API call, while
 * errors are unexpected issues that should not have happened at all.
 *
 * Basically this is the exception type that can be returned from the API
 * call safely.
 * ```typescript
 * const { res, err } = ... --> API call
 *
 * err; --> This `err` is `RequestException` type
 * ```
 *
 * ## Errors are not included
 * Error types are not part of this union, specifically `sfError`. `sfError`
 * will be **thrown** and not returned when the library is not correctly
 * configured / used.
 *
 * ## Library user defined exceptions are not included
 * When doing runtime response validation, if the validator throws a custom
 * exception type, e.g. user uses 'zod' and it throws a `ZodError`, this union
 * type will obviously not include that. However despite that, library users can
 * still check for that type using `err instanceof ZodError` in a type safe
 * manner, since instanceof operator can be used for narrowing even when it is
 * not defined on the union type itself.
 *
 * See this reference on all errors that can be thrown by the `fetch` API itself
 * https://developer.mozilla.org/en-US/docs/Web/API/fetch#exceptions
 */
type RequestException = 
// AbortError on timeout will throw a DOMException error, however there is a
// possibility that the error can be of DOMException type while not being an
// 'AbortError', which is why this exception type is still included.
DOMException | TypeError | TimeoutException | HeaderException | ValidationException;
/**
 * Expected return type of all `Fetch` run methods that do response parsing.
 *
 * Use satisfies to typecheck, to ensure that the return type will not be
 * generalized into the opaque `ApiResponse<T>` where users would not be able
 * to see the shape of the returned data clearly.
 */
type ApiResponse<T> = {
    ok: Response["ok"];
    status: Response["status"];
    headers: Response["headers"];
    data: T;
};
/**
 * Type of a generic type predicate, used for response validation.
 */
type Validator<T> = (data: unknown) => data is T;
/**
 * Function type for response parsing.
 */
type ResponseParser<T> = (res: Response) => Promise<T>;
/**
 * Class used to configure `fetch` request options with the builder pattern
 * using a simple chainable interface before finally making the API call itself
 * using one of the methods prefixed with `run`.
 *
 * This **SHOULD NOT** be used by library users directly, this should be
 * constructed using the `Builder` class.
 */
declare class Fetch {
    #private;
    readonly method: HTTPMethod;
    /**
     * Low level constructor API that should not be used by library users.
     * This is only used by the `Builder` class.
     */
    constructor(method: HTTPMethod, url: string, defaultOptions: RequestInit, defaultHeaders: Array<Header>);
    /**
     * Method to get the full generated URL. Use to get the full URL after
     * constructing it to use for things like reflecting back to the URL.
     *
     * This will generate the full URL including any search params used.
     */
    getUrl(): string;
    /**
     * Method to add Query Params to the final URL.
     *
     * Note that any query params set here will be merged/added to any existing
     * query params set via the URL Path string directly, and query params set
     * here will appear **after** the existing query params.
     *
     * The query params are also lazily merged, either when the fetch call is just
     * about to run and it calls `getUrl`, or if library user calls `getUrl`.
     *
     * @returns Returns the current instance to let you chain method calls
     */
    useQuery<T extends Record<string, string> = Record<string, string>>(queryParams: T): Fetch;
    /**
     * Method to use default `RequestInit` object of the selected base Url for
     * this specific API call.
     *
     * You can override any default options set through this method once off for
     * this specific API call with the `useOptions` method.
     *
     * @returns Returns the current instance to let you chain method calls
     */
    useDefaultOptions(): Fetch;
    /**
     * Method to use default header(s) of the selected base Url for this specific
     * API call.
     *
     * You can override any default headers set through this method once off for
     * this specific API call with the `useHeader` method.
     *
     * @returns Returns the current instance to let you chain method calls
     */
    useDefaultHeaders(): Fetch;
    /**
     * Method to set `RequestInit` object for this one API call.
     *
     * ### When to use this method?
     * Use this to set custom RequestInit parameters for this specific one-off API
     * call. See below on when should you use this method.
     *
     * Note that options set with this method cannot override the headers set with
     * the `useHeader` method, cannot override HTTP `method` via constructor and
     * cannot override the body value set by any of the 'body' methods.
     *
     * Which is why:
     * - If you want to set request body, use `.body` or `.bodyJSON` method.
     * - If you need to set a request header, use the `.useHeader` method.
     * - If you want to set HTTP request method, use a static method on `Builder`
     *
     * Do not use this unless you have a specific option to pass in e.g. cache:
     * "no-cache" for this one specific API call only and nothing else. If there
     * are options that you want to set for all API request calls, use `Builder`'s
     * `.setDefaultOptions` method instead. Note that any options value set using
     * this method will also override the default options set.
     *
     * This method merges the provided options with the previously set options, so
     * default options can be overridden there. Note that this is a shallow merge
     * and not a deepmerge.
     *
     * @returns Returns the current instance to let you chain method calls
     */
    useOptions(options: RequestInit): Fetch;
    /**
     * Add Header(s) to include in the API call.
     *
     * Accepts plain header objects, functions and async functions.
     *
     * Functions passed in will be called right before the API call to generate a
     * header object, to delay generating certain header values like a time
     * limited auth token or recaptcha.
     *
     * This method can be called multiple times, and all the header objects will
     * be combined. If there are duplicate headers, the latter will be used.
     *
     * If you prefer, this method is a variadic function that accepts multiple
     * arguments of the same type so that you can call this method just once if
     * you have all the headers instead of invoking this method multiple times.
     *
     * The function parameter forces the caller to pass in at least one argument
     * for this variadic method. See https://stackoverflow.com/a/72286990
     *
     * @returns Returns the current instance to let you chain method calls
     */
    useHeader(...headers: [
        Header,
        ...Header[]
    ]): Fetch;
    /**
     * Use this method to set a custom timeout, instead of relying on brower
     * default timeouts like Chrome's 300 seconds default.
     *
     * @returns Returns the current instance to let you chain method calls
     */
    timeoutAfter(timeoutInMilliseconds: number): Fetch;
    /* All methods for setting body starts with the word 'body' */
    /**
     * Set request body to be sent to server for HTTP methods such as POST/PUT.
     *
     * ### When to use this method?
     * For most users who want to send JSON data to the server, see the `bodyJSON`
     * method instead. For other types of data like `FormData`, `Blob`, `streams`
     * just pass it into this method as the `body` parameter and the content type
     * will be automatically detected / set by the `fetch` function.
     *
     * ### Why have both `body` and `bodyJSON` method?
     * The reason for this method instead of just having `bodyJSON` only is
     * because the library cannot always just assume that library users only use
     * JSON data, and have to support data types like FormData, Blob and etc...
     * However the problem is that when content-type is not set, `fetch` will try
     * to guess it, but when body is the results of `JSON.stringify(this.#body)`,
     * fetch will guess the content-type to be text/plain and the browser will
     * treat it as a safe CORS request, which means that for that request there
     * will be no pre-flight request sent. Which means that the browser prevents
     * certain headers from being used, which might cause an issue, and also the
     * server may not always respond correctly because they assume they got
     * text/plain even though your API endpoint is for application/json.
     *
     * The type of `body` value can be anything, as you can pass in any value that
     * the `fetch` API's `RequestInit`'s body property accepts.
     *
     * ### Using generics for TS Type Safety
     * ```typescript
     * const { res, err } = await sf
     *   .useDefault()
     *   .POST("/api")
     *   .body<FormData>(myValue) // TS will enforce that myValue is FormData
     *   .runJSON();
     * ```
     * The above code sets the body type for type safety.
     *
     * For TS users, the body type is a generic type variable even though its
     * default type for it is `any` so that you can use it to restrict the type
     * passed into the method. This allows you to enforce type safety where once a
     * generic type is set, you know that the value passed in for the `body`
     * parameter cannot be any other type.
     *
     * ### On `optionalContentType`'s type safety
     * Note on `optionalContentType`'s type: Although it is possible to create a
     * union type of all allowed string literals for the content-type header /
     * mime types, it is not very feasible as it is a very big list that will be
     * updated in the future. Therefore users are expected to make sure that any
     * string they pass is valid.
     * References on all supported content-type values:
     * - https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Type
     * - https://stackoverflow.com/questions/23714383/what-are-all-the-possible-values-for-http-content-type-header/48704300#48704300
     * - https://www.iana.org/assignments/media-types/media-types.xhtml
     *
     * @returns Returns the current instance to let you chain method calls
     */
    body<RequestBodyType = any>(body: RequestBodyType, optionalContentType?: string): Fetch;
    /**
     * @param data Any data type that is of 'application/json' type and can be
     * stringified by `JSON.stringify`.
     *
     * ### About
     * Method that stringifies a JSON stringifiable data type to use as the
     * request body, and sets the content-type to 'application/json'.
     *
     * ### What data type can be passed in?
     * Any JS value that can be JSON serialized with `JSON.stringify()`. See the
     * link on what type of data can be used.
     * [MDN link](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/JSON/stringify#description)
     *
     * ### What if you have no data to send?
     * Even though there is no default arguement, you do not have to call this
     * method with an empty object when using a method like `POST` as `fetch` and
     * API services will just treat it as an empty object by default.
     *
     * ### Why not just use the `body` method?
     * Since JSON is the most popular methods of communication over HTTP, this
     * method helps users to write less code by stringifying their JS object and
     * setting content-type header to 'application/json', instead of requiring
     * library users to write `.body(JSON.stringify(data), "application/json")`
     * every single time they want to send JSON data to their API servers.
     * This also allows library users to specify type of data object using the
     * method generic for type checking.
     *
     * ### Using generics for TS Type Safety
     * ```typescript
     * const { res, err } = await sf
     *   .POST("/api")
     *   .bodyJSON<IReqBody>(val) // TS will enforce that val must be IReqBody
     *   .run();
     * ```
     * The above code sets the body type for type safety.
     *
     * For TS users, the data parameter is a generic type even though its default
     * type is `any` so that you can use it to restrict the type passed into the
     * method. This allows you to enforce type safety where once a generic type is
     * set, you know that the value passed in for the `body` parameter cannot be
     * any other type.
     *
     * @returns Returns the current instance to let you chain method calls
     */
    bodyJSON<JsonRequestBodyType = JsonTypeAlias>(data: JsonRequestBodyType): Fetch;
    /**
     * # Warning
     * This method is generally not used since this returns the raw HTTP Response
     * object. Library users should use the other run methods that also handles
     * response parsing, such as `runJSON`, `runFormData` and etc... This method
     * is made available as an escape hatch for library users who do not want the
     * library to do response parsing for them, so that they can build extra logic
     * on top of this library by getting the raw Response object back.
     *
     * This is `safe`, i.e. this method **will not throw** or let exceptions
     * bubble up so no try/catch wrapper block or .catch method is needed to
     * handle the jumping control flow of exceptions.
     *
     * @example Call API and handle any exception sequentially in the same scope
     * ```typescript
     * const { res, err } = await sf.useDefault().GET("/api").run();
     *
     * if (err) return console.log("API Call failed!");
     *
     * console.log("Res:", res); // Type narrowed to be the Response object
     * ```
     */
    run(): Promise<{
        res: undefined;
        err: RequestException;
    } | {
        res: Response;
        err: undefined;
    }>;
    /**
     * Call API after configuring and get back response parsed as a **string**.
     *
     * This is `safe`, i.e. this method **will not throw** or let exceptions
     * bubble up so no try/catch wrapper block or .catch method is needed to
     * handle the jumping control flow of exceptions.
     *
     * @example Call API and handle any exception sequentially in the same scope
     * ```typescript
     * const { res, err } = await sf.useDefault().GET("/api").runText();
     *
     * if (err) return console.log("API Call failed!");
     *
     * console.log("Res:", res); // Type narrowed to be be a string
     * ```
     */
    runText<ErrorType>(optionalValidator?: Validator<string>, optionalErrorResponseParser?: ResponseParser<ErrorType>): Promise<{
        res: undefined;
        err: RequestException;
    } | {
        res: {
            ok: true;
            status: number;
            headers: Headers;
            data: string;
        } | {
            ok: false;
            status: number;
            headers: Headers;
            data: ErrorType;
        };
        err: undefined;
    }>;
    /**
     * Call API after configuring and get back response parsed as a **Blob**.
     *
     * This is `safe`, i.e. this method **will not throw** or let exceptions
     * bubble up so no try/catch wrapper block or .catch method is needed to
     * handle the jumping control flow of exceptions.
     *
     * @example Call API and handle any exception sequentially in the same scope
     * ```typescript
     * const { res, err } = await sf.useDefault().GET("/api").runBlob();
     *
     * if (err) return console.log("API Call failed!");
     *
     * console.log("Res:", res); // Type narrowed to be a Blob
     * ```
     */
    runBlob<ErrorType>(optionalValidator?: Validator<Blob>, optionalErrorResponseParser?: ResponseParser<ErrorType>): Promise<{
        res: undefined;
        err: RequestException;
    } | {
        res: {
            ok: true;
            status: number;
            headers: Headers;
            data: Blob;
        } | {
            ok: false;
            status: number;
            headers: Headers;
            data: ErrorType;
        };
        err: undefined;
    }>;
    /**
     * Call API after configuring and get back response parsed as **FormData**.
     *
     * This is `safe`, i.e. this method **will not throw** or let exceptions
     * bubble up so no try/catch wrapper block or .catch method is needed to
     * handle the jumping control flow of exceptions.
     *
     * @example Call API and handle any exception sequentially in the same scope
     * ```typescript
     * const { res, err } = await sf.useDefault().GET("/api").runFormData();
     *
     * if (err) return console.log("API Call failed!");
     *
     * console.log("Res:", res); // Type narrowed to be form data
     * ```
     */
    runFormData<ErrorType>(optionalValidator?: Validator<FormData>, optionalErrorResponseParser?: ResponseParser<ErrorType>): Promise<{
        res: undefined;
        err: RequestException;
    } | {
        res: {
            ok: true;
            status: number;
            headers: Headers;
            data: FormData;
        } | {
            ok: false;
            status: number;
            headers: Headers;
            data: ErrorType;
        };
        err: undefined;
    }>;
    /**
     * Call API after configuring and get back response parsed as **ArrayBuffer**.
     *
     * This is `safe`, i.e. this method **will not throw** or let exceptions
     * bubble up so no try/catch wrapper block or .catch method is needed to
     * handle the jumping control flow of exceptions.
     *
     * @example Call API and handle any exception sequentially in the same scope
     * ```typescript
     * const { res, err } = await sf.useDefault().GET("/api").runArrayBuffer();
     *
     * if (err) return console.log("API Call failed!");
     *
     * console.log("Res:", res); // Type narrowed to be an Array Buffer
     * ```
     */
    runArrayBuffer<ErrorType>(optionalValidator?: Validator<ArrayBuffer>, optionalErrorResponseParser?: ResponseParser<ErrorType>): Promise<{
        res: undefined;
        err: RequestException;
    } | {
        res: {
            ok: true;
            status: number;
            headers: Headers;
            data: ArrayBuffer;
        } | {
            ok: false;
            status: number;
            headers: Headers;
            data: ErrorType;
        };
        err: undefined;
    }>;
    /**
     * Call API after configuring and get back response parsed as **JSON**.
     *
     * This is `safe`, i.e. this method **will not throw** or let exceptions
     * bubble up so no try/catch wrapper block or .catch method is needed to
     * handle the jumping control flow of exceptions.
     *
     * @example Call API and handle any exception sequentially in the same scope
     * ```typescript
     * const { res, err } = await sf.useDefault().GET("/api").runJSON<MyResponseObjectType>();
     *
     * if (err) return console.log("API Call failed!");
     *
     * console.log("Res:", res); // Type narrowed to be 'MyResponseObjectType'
     * ```
     *
     * ### Using generics for TS Type Safety
     * This method accepts a generic type to type the returned object. Allowing
     * you to have type safety for the response object. However, this DOES NOT
     * perform any runtime response validation, so even if it is type safe, it
     * does not mean that the response object is guaranteed to be what you typed
     * it to be. For run time type safety, please pass in a validator (type
     * predicate) to do runtime response data validation, so that the library can
     * safely type narrow it down to the generic type `SuccessType` passed in.
     *
     * `SuccessType` generic defaults to unknown, to force library users to either
     * specify a generic type or type narrow the return data themselves after
     * getting it back. If you do not want to type narrow it yourself and want to
     * make it `any` instead, you need to explicitly use `any` for the generic.
     *
     * However for `ErrorType` generic, it default to `SuccessType`. Why? This is
     * because a library user might choose to create their own union type for both
     * the success data and error data types, and use it as a single type before
     * using a type discriminant to discrimiate between them, rather than relying
     * on this library's `ApiResponse.ok` as the type discriminant.
     *
     * If function signature is `runJSON<SuccessType, ErrorType>`, it means that
     * if the user only wants to pass in a single type, they can't do so since
     * both have to be either unspecified or specified at the same time.
     *
     * If function signature is `runJSON<SuccessType, ErrorType = unknown>`, it
     * means that although users can just pass in a single type, they cannot pass
     * in their own union types since the library will treat API data with
     * `ApiResponse.ok === false` as unknown instead, which prevents them from
     * using a unified type.
     *
     * This behavior matches the other run methods where all of their `ErrorType`
     * generics defaults to the same fixed success type.
     */
    runJSON<SuccessType, ErrorType = SuccessType>(optionalValidator?: Validator<SuccessType>, optionalErrorResponseParser?: ResponseParser<ErrorType>): Promise<{
        res: undefined;
        err: RequestException;
    } | {
        res: {
            ok: true;
            status: number;
            headers: Headers;
            data: SuccessType;
        } | {
            ok: false;
            status: number;
            headers: Headers;
            data: ErrorType;
        };
        err: undefined;
    }>;
    /**
     * Call API after configuring and get back response as **null**. This is
     * useful for calling APIs that you know explicitly returns nothing such as a
     * POST request that returns just a 201 status code. By using this method and
     * having the response data be `null` to represent void, it prevents
     * accidental access of any methods that should not be accessed.
     *
     * This is `safe`, i.e. this method **will not throw** or let exceptions
     * bubble up so no try/catch wrapper block or .catch method is needed to
     * handle the jumping control flow of exceptions.
     *
     * @example Call API and handle any exception sequentially in the same scope
     * ```typescript
     * const { res, err } = await sf.useDefault().GET("/api").runVoid();
     *
     * if (err) return console.log("API Call failed!");
     *
     * console.log("Res:", res); // Res: null
     * ```
     */
    runVoid<ErrorType>(optionalErrorResponseParser?: ResponseParser<ErrorType>): Promise<{
        res: undefined;
        err: RequestException;
    } | {
        res: {
            ok: true;
            status: number;
            headers: Headers;
            data: null;
        } | {
            ok: false;
            status: number;
            headers: Headers;
            data: ErrorType;
        };
        err: undefined;
    }>;
}
/**
 * Class used to implement the Builder pattern for creating `Fetch` instances.
 * This class should only be used internally by `sf` and should not be exposed
 * to library users since they should not be using this to make API calls.
 */
declare class Builder {
    #private;
    constructor(baseUrl: string);
    /**
     * Method to set default options that will be applied to all API calls made
     * through the `Fetch` instance created by this `Builder` instance.
     * The options set here can be overridden one-off in specific API calls with
     * the `useOptions` method of `Fetch` class.
     *
     * Note that everytime this is called, all the default options are overwritten
     * to use the options passed in as the new default options without merging
     * with the original default options. Note that this behavior **CANNOT** be
     * changed since the `HTTP` method's `Fetch` instantiation relies on this.
     * This is to ensure that any modifications to the default values will not
     * affect the already instantiated `Fetch` instances, since the `Fetch`
     * instances will still hold on to the original container value's reference.
     * This prevent certain security risks such as dynamically changing the
     * `credentials` property to include it even when the original default option
     * is to omit it.
     * See commit f947ec0b590286b45dfccfc6229dba3550411c6a
     *
     * @returns Returns the current instance to let you chain method calls
     */
    setDefaultOptions(opts: RequestInit): Builder;
    /**
     * Method to set default headers that will be applied to all API calls made
     * through the `Fetch` instance created by this `Builder` instance.
     * The headers set here can be overwritten one-off in specific API calls with
     * the `useHeader` method of `Fetch` class.
     *
     * Useful for doing things like passing in a function to generate auth header,
     * so that the function does not need to be passed in for every API call made
     * with the `Fetch` instance. Another example is if every API call needs a
     * recaptcha token, instead of passing in the recaptcha token header generator
     * function everywhere, this can be set just once as a default header value.
     *
     * Note that everytime this is called, the default headers is overwritten
     * to use the headers passed in as the new default headers without merging
     * with the original default headers. Note that this behavior **CANNOT** be
     * changed since the `HTTP` method's `Fetch` instantiation relies on this.
     * This is to ensure that any modifications to the default values will not
     * affect the already instantiated `Fetch` instances, since the `Fetch`
     * instances will still hold on to the original container value's reference.
     * This prevent certain security risks such as dynamically changing the
     * certain sensitive header values.
     * See commit f947ec0b590286b45dfccfc6229dba3550411c6a
     *
     * The `headers` function parameter forces the caller to pass in at least one
     * argument for this variadic method. See https://stackoverflow.com/a/72286990
     *
     * @returns Returns the current instance to let you chain method calls
     */
    setDefaultHeaders(...headers: [
        Header,
        ...Header[]
    ]): Builder;
    /**
     * tl;dr Do not use this unless you know what you are doing.
     *
     * ### About
     * Wrapper over `Fetch` constructor to construct a new instance with the given
     * HTTP method and path alongside the values set on this `Builder` instance.
     *
     * ### Library Users
     * For the most part, library users ***SHOULD NOT*** be using this low level
     * API, instead they should use the methods named after the HTTP methods such
     * as `.GET`, `.POST` and so on. The only reason why this is made public is
     * for advanced library users who want to use HTTP methods like `HEAD` and
     * `OPTION` to still be able to do so using our library instead of doing some
     * crazy hack, since there is no method for those specific HTTP methods.
     */
    HTTP: (method: HTTPMethod, path?: string) => Fetch;
    /** Construct a new `Fetch` instance to make a `GET` API call */
    GET: (path?: string) => Fetch;
    /** Construct a new `Fetch` instance to make a `POST` API call */
    POST: (path?: string) => Fetch;
    /** Construct a new `Fetch` instance to make a `PUT` API call */
    PUT: (path?: string) => Fetch;
    /** Construct a new `Fetch` instance to make a `PATCH` API call */
    PATCH: (path?: string) => Fetch;
    /** Construct a new `Fetch` instance to make a `DELETE` API call */
    DEL: (path?: string) => Fetch;
}
/**
 * `sf` (simpler-fetch) is used to create an Object Oriented `Fetch` abstraction
 * to use the builder pattern to give users an easy to use chainable interface
 * to build their API calls.
 *
 * ## On Safety and Error handling
 * Note that although this library talks about safety, where errors are returned
 * instead of thrown, there is an exception to this case. All static methods on
 * this class is not safe since it can throw errors if anything goes wrong.
 *
 * Specifically, it throws the `sfError` class.
 *
 * The reason why this throws is to give high visibility to the issue because,
 * if there is an error here, it SHOULD NOT continue. This error will be thrown
 * when there is a library user caused configuration error and not a network/API
 * call related errors.
 *
 * This library is only safe when dealing with network/API call related errors.
 * It is advised to not wrap this in a try/catch block and just let the error
 * propagate so that you can see it and fix it before this goes to production!
 */
declare class sf {
    #private;
    /**
     * Add a new baseUrl identifier and baseUrl mapping.
     * For example, you can add different baseUrls for your APIs such
     * as 'v1', 'v2', 'latest', 'billing' and etc...
     */
    static addBase(identifier: string, url: string): typeof sf;
    /**
     * Select a baseUrl to use, using a baseUrl identifier.
     * For example, you can select 'v1' baseUrl after adding it with `sf.addBase`
     */
    static useBase(identifier: string): Builder;
    /**
     * Set a particular identifier and baseUrl mapping as the default mapping to
     * use so that you can easily use this default mapping in your API calls
     * without having to import and specify the identifier.
     */
    static setDefault(identifier: string): void;
    /**
     * Use the default identifier and baseUrl mapping for an API call.
     */
    static useDefault(): Builder;
    /**
     * Use a Full URL string (with API path) to make an API call without using any
     * baseUrl.
     *
     * This method should be used when making one off API calls, without having to
     * set a `baseUrl` first. Usually used when you need to make an API call to
     * another domain. E.g. calling a third party API for some integration.
     */
    static useOnce: (fullUrlString: string) => Builder;
    /**
     * Utilities attached to `sf` so that users can access these without manually
     * importing them separately and potentially dealing with naming conflicts.
     */
    static readonly utils: {
        skipValidation: <T>(_: unknown) => _ is T;
        jsonParser: ResponseParser<any>;
    };
}
/**
 * Custom named error class so that library users can check for this failure
 * mode with the `instanceof` operator.
 *
 * This error will be exclusively by the `sf` class when there is an error,
 * usually caused by configuration issues.
 *
 * Note that this error will not be returned by the API call, instead this will
 * be an error ***thrown***! This is thrown to give high visibility to the issue
 * since this is a library user error and not a network/API call related error.
 */
declare class sfError extends Error {
}
/**
 * `jsonParser` utility function is used to parse Response data as json.
 */
declare const jsonParser: ResponseParser<any>;
/**
 * Use this when validator function is required but you do not want to provide
 * one because it might not really matter to your use case.
 */
declare const skipValidation: <T>(_: unknown) => _ is T;
/**
 * Utility to convert a zod parser to validator function for validating response
 * data in `Fetch` run methods.
 *
 * Alternative conversion using the zod `safeParse` method instead.
 * The problem with this, is that no exceptions are thrown from the parser,
 * which means that the API lib will always return its own Exception instance
 * with a Validation Failed exception message, while it might be preferrable for
 * users to receive the actual `ZodError` by having it bubble through our API
 * library. So that users can get it with `const { res, err } =` and they can
 * check `err` type using `instanceof`.
 *
 * Alternative using safeParse
 * ```typescript
 * const zodToValidator =
 *   <ZodParserType extends ZodType>(parser: ZodParserType) =>
 *   (data: unknown): data is z.infer<ZodParserType> =>
 *     parser.safeParse(data).success;
 * ```
 */
declare const zodToValidator: <ZodParserType extends z.ZodType<any, z.ZodTypeDef, any>>(parser: ZodParserType) => Validator<z.TypeOf<ZodParserType>>;
/**
 * Barrel file to export everything a library user can access.
 */
export { sf, sfError, TimeoutException, HeaderException, ValidationException, jsonParser, skipValidation, zodToValidator, HeaderValue, Header, HTTPMethod, JsonTypeAlias, RequestException, ApiResponse, Validator, ResponseParser };
